/***
*
* File.......: FileLog.ch
*
* Author.....: John Van Ostrand, (c) Copyright 1992
*
* Description: Contains all of the nessecary functions to log used database
*              and index file names.
*
* Notes......: For use with Nantucket's Clipper v5.01
*              Compile using CLIPPER FileLog /N/A/W
*
***/

// Define some handy values.
#Define FL_WORK_AREA    1
#Define FL_DBF_NAME   2

/***
*   Static Variable: DbfNtx
*
*   Each element of the array DbfNtx will contain;
*
*   Structure:   { Work Area,;
*                  Database Name,;
*                  [Index Name 1, ];
*                  ...
*                  [Index Name n ,]
*                }
*
***/
Static DbfNtx := {}



/***
*DICT DBFLOG()           Logs a newly opened database.
*
* ^BSyntax;
*
*       ^BDBFLOG( <cDbfName> ) => NIL
*
* ^BArguments;
*
*       ^B<cDbfName>^B is the  name of the database file that was opened.
*
* ^BReturns;
*
*       DBFLOG() always returns NIL.
*
* ^BDescription;
*
*       DBFLOG() is used to log opened databases. It is to be used in
*       conjunction with NTXLOG() to record opened index files for use with
*       any functions that require index names or require knowledge of the
*       work areas in use.
*
*       NOTE: The file FILELOG.CH contains all of the nessecary #COMMAND
*       statements to log opened files when using any of the following
*       commands;
*
*               USE
*               CLOSE
*               INDEX ON ... TO
*               SET INDEX TO
*
*       If the associated database functions are used instead of the above
*       commands, the file logging functions must also be used in order to
*       maintain the log integrity. This particular function is for use with
*       the USE and CLOSE commands or the DbUSeArea() and DbCloseArea()
*       functions.
*ENDD
***/


Function DbfLog(cDbfName, nWorkArea)
Local nElement

    If(nWorkArea == NIL, nWorkArea := Select(), )

    // Check to see if we are reusing the work area.
    nElement := AScan(DbfNtx, { |aEl| aEl[FL_WORK_AREA] == nWorkArea } )

    // If the work area is being reused then just an assignment is nessecary.
    If nElement != 0
        DbfNtx[nElement] := {nWorkArea, cDbfName}
    Else
        // Otherwise we add an element.
        Aadd(DbfNtx, {nWorkArea, cDbfName})
    EndIf
Return ( NIL )



/***
*DICT NTXLOG()           Logs a newly opened index.
*
* ^BSyntax;
*
*       ^BNTXLOG( <cNtxName> ) => NIL
*
* ^BArguments;
*
*       ^B<cNtxName>^B is the  name of the index file that was opened.
*
* ^BReturns;
*
*       NTXLOG() always returns NIL.
*
* ^BDescription;
*
*       NTXLOG() is used to log opened indexes. It is to be used in
*       conjunction with DBFLOG() to record opened index files for use with
*       any functions that require index names or require knowledge of the
*       work areas in use.
*
*       NOTE: The file FILELOG.CH contains all of the nessecary #COMMAND
*       statements to log opened files when using any of the following
*       commands;
*
*               USE
*               CLOSE
*               INDEX ON ... TO
*               SET INDEX TO
*
*       If the associated database functions are used instead of the above
*       commands, the file logging functions must also be used in order to
*       maintain the log integrity. This particular function is for use with
*       the USE command or the DbSetIndex() function.
*ENDD
***/


Function NtxLog(cNtxName, nWorkArea)
Local nElement

    // Resolve for the optional parameter.
    If(nWorkArea == NIL, nWorkArea := Select(), )

    // Find the proper work area.
    nElement := AScan(DbfNtx, { |aEl| aEl[FL_WORK_AREA] == nWorkArea } )

    // Add the index name to the array.
    Aadd(DbfNtx[nElement], cNtxName)

Return ( NIL )


/***
*DICT DBFLOSELOG()   Logs a newly opened index.
*
* ^BSyntax;
*
*       ^BDBFCLOSELOG( [<nWorkArea>] ) => NIL
*
* ^BArguments;
*
*       ^B<nWorkArea>^B is the work area of the closed database. If omitted
*       the current area is used.
*
* ^BReturns;
*
*       DBFCLOSELOG() always returns NIL.
*
* ^BDescription;
*
*       DBFCLOSELOG() is used to remove logged files. Any files whether
*       database or index are removed from the log.
*
*       NOTE: The file FILELOG.CH contains all of the nessecary #COMMAND
*       statements to log opened files when using any of the following
*       commands;
*
*               USE
*               CLOSE
*               INDEX ON ... TO
*               SET INDEX TO
*
*       If the associated database functions are used instead of the above
*       commands, the file logging functions must also be used in order to
*       maintain the log integrity. This particular function is for use with
*       the USE and CLOSE commands or the DbUseArea() and DbCloseArea()
*       functions.
*ENDD
***/


Function DbfCloseLog(nWorkArea)
Local nElement

    // Reslove for the optional parameter.
    If(nWorkArea == NIL, nWorkArea := Select(), )

    // Find the associated element.
    nElement := AScan(DbfNtx, { |aEl| aEl[FL_WORK_AREA] == nWorkArea } )

    // If the element was found then we can remove it.
    If nElement > 0
        ADel(DbfNtx, nElement)
        ASize(DbfNtx, Len(DbfNtx) - 1)
    EndIf

Return ( NIL )



/***
*DICT NTXCLOSELOG()   Logs a newly opened index.
*
* ^BSyntax;
*
*       ^BNTXCLOSELOG( [<nWorkArea>] ) => NIL
*
* ^BArguments;
*
*       ^B<nWorkArea>^B is the work area of the closed indexes. If omitted
*       the current area is used.
*
* ^BReturns;
*
*       NTXCLOSELOG() always returns NIL.
*
* ^BDescription;
*
*       NTXCLOSELOG() is used to remove logged files. Any index files for the
*       specified work area removed from the log.
*
*       NOTE: The file FILELOG.CH contains all of the nessecary #COMMAND
*       statements to log opened files when using any of the following
*       commands;
*
*               USE
*               CLOSE
*               INDEX ON ... TO
*               SET INDEX TO
*
*       If the associated database functions are used instead of the above
*       commands, the file logging functions must also be used in order to
*       maintain the log integrity. This particular function is for use with
*       the USE and CLOSE commands or the DbCloseArea() and DbClearIndex()
*       functions.
*ENDD
***/


Function NtxCloseLog(nWorkArea)
Local nElement

    // Reslove for the optional parameter.
    If(nWorkArea == NIL, nWorkArea := Select(), )

    // Find the associated element.
    nElement := AScan(DbfNtx, { |aEl| aEl[FL_WORK_AREA] == nWorkArea } )

    // If the element was found then we can remove all index logs.
    If nElement > 0
        DbfNtx[nElement] := {DbfNtx[nElement, FL_WORK_AREA], ;
                                DbfNtx[nElement, FL_DBF_NAME]}
    EndIf

Return ( NIL )


/***
*DICT NTX()          Retreive the name of an index file.
*
* ^BSyntax;
*
*       NTX( [<nIndexNumber>], [<nWorkArea>] ) => cNtxName
*
* ^BArguments;
*
*       ^B<nIndexNumber>^B is the number of the index. If omitted the
*       active index is used.
*
*       ^B<nWorkArea> is the work area number. If omitted the current work
*       area is used.
*
* ^BReturns;
*
*       NTX() returns the name of the specified index or "" if no index
*       exists. The index name returned is the same name as specifed in
*       the USE, SET INDEX, or INDEX ON ... TO commands. This usually means
*       that the '.NTX' extension is omitted.
*
* ^BDescription;
*
*       NTX() ats much like the DBF() or ALIAS() function except that NTX()
*       returns the name of a specified index.
*
*ENDD
***/

Function Ntx(nIndexNumber, nWorkArea)
Local nElement, cIndexName := ""

    // Resolve for the two optional parameters.
    If(nWorkArea == NIL, nWorkArea := Select(), )
    If(nIndexNumber == NIL, nIndexNumber := (nWorkArea)->(IndexOrd()), )

    // Find the appropriate array element.
    nElement := AScan(DbfNtx, { |aEl| aEl[FL_WORK_AREA] == nWorkArea } )

    // If the element exists and contains the index name ...
    If nElement > 0 .and. Len(DbfNtx[nElement]) >= nIndexNumber + 2
        // then set the return value.
        cIndexName := DbfNtx[nElement, nIndexNumber + 2]
    EndIf
Return ( cIndexName )


/***
*DICT DBFKILLOG()    Removes all database and index logs.
*
* ^BSyntax;
*
*       DBFKILLLOG() => NIL
*
* ^BReturns;
*
*       DBFKILLLOG() always returns NIL.
*
* ^BDescription;
*
*       DBFKILLLOG() removes all entries from the log. This would be used
*       to log a complete closure of all databases.
*
*       NOTE: The file FILELOG.CH contains all of the nessecary #COMMAND
*       statements to log opened files when using any of the following
*       commands;
*
*               USE
*               CLOSE
*               INDEX ON ... TO
*               SET INDEX TO
*
*       If the associated database functions are used instead of the above
*       commands, the file logging functions must also be used in order to
*       maintain the log integrity. This particular function is for use with
*       the CLOSE command or the DbCloseAll() function.
*ENDD
***/

Function DbfKillLog()

    // Completely remove all log information.
    DbfNtx := {}
Return ( NIL )


/***
*DICT FILELOG()      Retreive the file log in its entirety.
*
* ^BSyntax;
*
*       ^BFILELOG() => aFileLog
*
* ^BReturns;
*
*       FILELOG() returns an array containing a record of all the databases,
*       their workareas and their associated index files. The array is
*       structured as follows;
*
*       Each element of the file log array contains;
*
*           {
*               Work Area Number ,;
*               Database Name    ,;
*               Index Name #1    ,;
*               Index Name #2    ,;
*               ...              ,;
*               Index Name #n
*           }
*
* ^BDescription;
*
*       FILELOG() returns an array containing all of the logged databases,
*       their work areas and their associated indexes.  This may be useful
*       in determining the highest use work area or for evaluating a code
*       block for each database or index.
*
* ^BExample;
*
*       ^B// Define som handy constants.
*       ^B#DEFINE FL_WORK_AREA      1
*       ^B#DEFINE FL_DBF_NAME       2
*
*       ^B// Find the highest used work area.
*       ^BnMaxArea := 0
*       ^BAEVAL( FILELOG(), ;
*       ^B            { |aEl| nMaxArea := MAX( nMaxArea, aEl[FL_WORK_AREA] } )
*
*       ^B// Print the list of used work areas, databases and indexes.
*       ^BAEVAL( FILELOG(), ;
*       ^B            { |aEl| AEVAL( aEl, { |El| QQOUT( El ) } ), QOUT() } )
*
*ENDD
***/

Function FileLog()

    // Just return the array reference.
Return ( DbfNtx )
