/***
*
*   Author: John Van Ostrand
*
*   Functions:
*       MakeBar(nSRow, nSCol, nERow, nViewSize, nInitPos, cChars, ;
*           cArrowColor, cBackColor, cPosColor) => aScrolBarVariable
*
*       DisplayBar(aScrollBarVariable) => NIL
*
*       UpdateBar(aScrollBarVariable, nCurItem, nTotal, lForce) => NIL
*
***/


/***
*DICT MAKEBAR()      Creates a scroll bar indicator variable.
*
* ^BSyntax
*
*       ^bMAKEBAR( <nSRow>, <nSCol>, <nERow>, <nViewSize>, [<nInitPos>],
*           ^B[<cChars>], [<cArrowColor>], [<cBackColor>],
*           ^B[<cPosColor>] ) --> aScrollBarVariable
*
* ^BArguments
*
*       ^b<nSRow>^B is the top row of the scroll bar.
*
*       ^B<nSCol>^b is the column that the scroll bar will be displayed.
*
*       ^B<nERow>^B is the bottom row of the scroll bar.
*
*       ^B<nViewSize>^B is the size of the data display window.
*
*       ^B<nInitPos>^B is the initial position of the indicator. The default
*       value for this argument is 0.
*
*       ^B<cChars>^B is a string containing the scroll bar characters. The
*       default value for this argument is "".
*
*       ^B<cArrowColor>^B is the color of the scroll bar arrows. The default
*       value for this argument is the current color.
*
*       ^B<cBackColor)>^B is the color of the remaining scroll bar. The default
*       value for this argument is the current color.
*
*       ^B<cPosColor>^B is the color of the view indicator. The default
*       value for this argument is the current color.
*
* ^BReturns
*
*       MAKEBAR() returns an array containing information needed by the
*       other scroll bar function to display and update the scroll bar.
*
* ^BDescription
*
*       This function sets up a scroll bar variable for use with the
*       other scroll bar functions.
*CALLS DISPLAYBAR() UPDATEBAR()
*ENDD
***/

Function MakeBar(SRow, SCol, ERow, ViewSize, InitPos, Chars, ArrowColor, BackColor, PosColor)
Local SBarVals := Array(9)
    SBarVals[1] := SRow
    SBarVals[2] := SCol
    SBarVals[3] := ERow
    SBarVals[4] := ViewSize
    SBarVals[5] := If(InitPos == NIL, 0, InitPos)
    SBarVals[6] := If(Chars == NIL,Chr(30)+Chr(31)+Chr(176)+Chr(177), Chars)
    SBarVals[7] := If(ArrowColor == NIL, SetColor(), ArrowColor)
    SBarVals[8] := If(BackColor == NIL, SetColor(), BackColor)
    SBarVals[9] := If(PosColor == NIL, SetColor(), PosColor)
Return SBarVals


/***
*DICT DISPLAYBAR()   Displays a scroll bar without the iundicator.
*
* ^BSyntax
*
*       ^BDISPLAYBAR( <aScrollBarVaraible> ) --> NIL
*
* ^BArguments
*
*       <aScrollVariable> is a scroll bar varaible returned by MAKEBAR().
*
* ^BDescription
*       This function displays a scroll bar. The indicator is not
*       displayed.
*CALLS MAKEBAR() UPDATEBAR()
*ENDD
***/

Function DisplayBar(SBar)
Local x
    If SetScrBar()
        @ SBar[1], SBar[2] Say Substr(SBar[6], 1, 1) Color SBar[7]
        For x = SBar[1] + 1 to SBar[3] -1
            @ x, SBar[2] Say SubStr(Sbar[6], 3, 1) Color SBar[8]
        Next
        @ SBar[3], SBar[2] Say SubStr(SBar[6], 2, 1) Color SBar[7]
    EndIf
Return NIL



/***
*DICT UPDATEBAR()    Displays the indicator of a scroll bar.
*
* ^BSyntax
*
*       ^BUPDATEBAR( <aSBar>, <nCurItem>,
*           ^B<nTotal>, [<lForce>] ) --> NIL
*
* ^BArguments
*
*       ^B<aSBar>^B is a scroll bar variable created by MAKEBAR().
*
*       ^B<nCurItem>^B is the item number that is currently at the top of the
*       display window.
*
*       ^B<nTotal>^B is the total number of items in the list.
*
*       ^B<lForce>^B tells UPDATEBAR() to force the display of the indicator. The
*       default value is .F. (False).
*
* ^BDescription
*
*       This function displays the scroll bar indicator. Force is the
*   only optional parameter.  It forces the display of the indicator.
*   If the indicator has not changed position it is not redisplayed unless
*   ^B<lForce>^B is set to .T. (True).
*       ^B<nCurItem>^B indicates the upper most row of the current view and
*   ^B<nTotal>^B indicates the total number of rows in the list.
*CALLS MAKEBAR() DISPLAYBAR()
*ENDD
***/

Function UpDateBar(SBar, CurItem, Total, Force)
Local NoBlock, BlockSize
Local Length := SBar[3] - SBar[1] - 1
Local OldRow := Row(), OldCol := Col()
Local CurInd

    If SetScrBar()
        CurItem := CurItem - 1
        Total := Total - 1
        BlockSize := If(SBar[4] > Total, Length, (SBar[4] / Total) * Length)
        NoBlock := If(Round(BlockSize,0) = 0, 1, Round(BlockSize, 0))
        Force := If(Force == NIL, .F., Force)
        CurInd := Round(CurItem / SBar[4] * BlockSize, 0)
        CurInd := If(CurInd < 0, 0, If(CurInd+NoBlock > Length, Length - NoBlock, CurInd))
        If CurInd <> SBar[5] .or. Force
            SBar[5] := If(SBar[5] < 0, 0, If(SBar[5]+NoBlock > Length, Length - NoBlock, SBar[5]))
            DrawIndicator(SBar, NoBlock, .T.)
            SBar[5] := CurInd
            DrawIndicator(SBar, NoBlock, .F.)
        EndIf
        SetPos(OldRow, OldCol)
    EndIf
Return NIL


Static Function DrawIndicator(SBar, NoBlock, Clear)
Local x
    For x = 1 to NoBlock
        @ SBar[1] + SBar[5] + x, SBar[2];
            Say If(Clear, SubStr(SBar[6],3,1), SubStr(Sbar[6], 4, 1)) Color If(Clear, SBar[8], SBar[9])
    Next
Return NIL

/***
*
*DICT SETSCRBAR()    Toggles scroll bar display on or off.
*
* ^BSyntax
*
*       ^BSETSCRBAR( [<lSetting>] ) --> lCurrentSetting
*
* ^BArguments
*
*       <lSetting> if .T. (True) allows scroll bars to be displayed.
*       If .F. (False) scroll bars are suppressed.
*
* ^BReturns
*
*       SETSCRBAR() returns the current setting.
*
* ^BDescription
*
*       SETSCRBAR() is used to control the display of scroll bars.
*       If SETSCRBAR() is passed a .T. scroll bars are displayed, if
*       passed a .F. scroll bars are not displayed.
*CALLS GETMSG()
*ENDD
***/


Function SetScrBar(lSetting)
Static ScrBarStatus := .T.
Local OldSetting := ScrBarStatus

    If lSetting != NIL
        ScrBarStatus := lSetting
    EndIf
Return ( OldSetting )





